#
#  utex/pipe.rb  --  Pipe to TeX
#


require "utex/filter"
require "readline"


module UTeX

  class Pipe

    class <<self
      def open *args
        i = new *args
        yield i
      end
      def interactive *args
        open *args do |i| i.interactive end
      end
    end

    def initialize *args
      rd, wd = IO.pipe
      if (@ch = fork) then
        rd.close
        @out = Filter.new wd
      else
        wd.close ; $stdin.reopen rd ; rd.close
        exec "tex", *args.flatten
      end
      @out.putline "\\relax"
    end

    def interactive
      while running? and (l = readline) do @out.putline l end
    end

    private

    def readline
      l = Readline::readline
      Readline::HISTORY.push l unless not l or l.empty?
      l
    end

    def running?
      not Process.waitpid @ch, Process::WNOHANG
    end

  end

end

